﻿#region MIT Licenze
//The MIT License

//Copyright (c) 2008 Sebastian Faltoni, dotnetfireball.org

//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:

//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.

//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Fireball.ImagePak
{
    public class ImagePackageInfo
    {      
        public Uri MainImage
        {
            get;
            set;
        }

        public string Author
        {
            get;
            set;
        }

        public string Description
        {
            get;
            set;
        }

        public XElement ToXml()
        {
            XElement pkInfo = new XElement(NS.InfoNameSpace + "pkInfo",
                new XAttribute(XNamespace.Xmlns + "x", NS.InfoNameSpace.NamespaceName));

            pkInfo.Add(new XElement(NS.InfoNameSpace + "mainImage",
                 new XAttribute(NS.InfoNameSpace + "val",
                 this.MainImage)));

            if (!string.IsNullOrEmpty(this.Author))
                pkInfo.Add(new XElement(NS.InfoNameSpace + "author", this.Author));

            if (!string.IsNullOrEmpty(this.Description))
                pkInfo.Add(new XElement(NS.InfoNameSpace + "description", this.Description));

            return pkInfo;
        }


        private void ParseInternal(XElement pkInfo)
        {
            var mainImage = pkInfo.Element(NS.InfoNameSpace + "mainImage");

            if (this.MainImage == null)
                throw new Exception("Invalid Main Image Uri.");

            this.MainImage = new Uri(mainImage.Value);

            var author = pkInfo.Element(NS.InfoNameSpace + "author");

            if (author != null)
            {
                this.Author = author.Value;
            }

            var description = pkInfo.Element(NS.InfoNameSpace + "description");

            if (description != null)
            {
                this.Description = description.Value;
            }
        }

        public static ImagePackageInfo Parse(XElement pkInfo)
        {
            ImagePackageInfo info = new ImagePackageInfo();
            info.ParseInternal(pkInfo);
            return info;
        }
    }
}
